--[[
Clock Rings by londonali1010 (2009) Edited by jpope

This script draws percentage meters as rings, and also draws clock hands if you want! It is fully customisable; all options are described in the script. This script is based off a combination of my clock.lua script and my rings.lua script.

IMPORTANT: if you are using the 'cpu' function, it will cause a segmentation fault if it tries to draw a ring straight away. The if statement on line 145 uses a delay to make sure that this doesn't happen. It calculates the length of the delay by the number of updates since Conky started. Generally, a value of 5s is long enough, so if you update Conky every 1s, use update_num>5 in that if statement (the default). If you only update Conky every 2s, you should change it to update_num>3; conversely if you update Conky every 0.5s, you should use update_num>10. ALSO, if you change your Conky, is it best to use "killall conky; conky" to update it, otherwise the update_num will not be reset and you will get an error.

To call this script in Conky, use the following (assuming that you save this script to ~/scripts/rings.lua):
    lua_load ~/scripts/clock_rings.lua
    lua_draw_hook_pre clock_rings
    
Changelog:
+ v1.0 -- Original release (30.09.2009)
   v1.1p -- Jpope edit (05.10.2009)
modified by putputput (13.10.2010v0.0114.10.2010v0.02)
]]

settings_table = {
    {
        -- Edit this table to customise your rings.
        -- You can create more rings simply by adding more elements to settings_table.
        -- "name" is the type of stat to display; you can choose from 'cpu', 'memperc', 'fs_used_perc', 'battery_used_perc'.
        name='time',
        -- "arg" is the argument to the stat type, e.g. if in Conky you would write ${cpu cpu0}, 'cpu0' would be the argument. If you would not use an argument in the Conky variable, use ''.
        arg='%I.%M',
        -- "max" is the maximum value of the ring. If the Conky variable outputs a percentage, use 100.
        max=12,
        -- "bg_colour" is the colour of the base ring.
        bg_colour=0xffffff,
        -- "bg_alpha" is the alpha value of the base ring.
        bg_alpha=0.1,
        -- "fg_colour" is the colour of the indicator part of the ring.
        fg_colour=0x3399cc,
        -- "fg_alpha" is the alpha value of the indicator part of the ring.
        fg_alpha=0.2,
	--Լ޸ĵĽ0 1 ɫ2 ͸ȱ仯3 124 ߿5 ɫ͸ȹɣ6 45
	change_color=4,	
	start_alpha=0.2,
	end_alpha=0.2,
	startcolor=0x3399cc,
	endcolor=0x3399cc,
        -- "x" and "y" are the x and y coordinates of the centre of the ring, relative to the top left corner of the Conky window.
        x=100, y=170,
        -- "radius" is the radius of the ring.
        radius=50,
        -- "thickness" is the thickness of the ring, centred around the radius.
        thickness=5,
        -- "start_angle" is the starting angle of the ring, in degrees, clockwise from top. Value can be either positive or negative.
        start_angle=0,
        -- "end_angle" is the ending angle of the ring, in degrees, clockwise from top. Value can be either positive or negative, but must be larger than start_angle.
        end_angle=360
    },
    {
        name='time',
        arg='%M.%S',
        max=60,
        bg_colour=0xffffff,
        bg_alpha=0.1,
        fg_colour=0x3399cc,
        fg_alpha=0.4,
	change_color=4,	
	start_alpha=0.4,
	end_alpha=0.4,
	startcolor=0x3399cc,
	endcolor=0x3399cc,
        x=100, y=170,
        radius=56,
        thickness=5,
        start_angle=0,
        end_angle=360
    },
    {
        name='time',
        arg='%S',
        max=60,
        bg_colour=0xffffff,
        bg_alpha=0.1,
        fg_colour=0x3399cc,
        fg_alpha=0.6,
	change_color=6,	
	start_alpha=0,
	end_alpha=0.6,
	startcolor=0x3399cc,
	endcolor=0x3399cc,
        x=100, y=170,
        radius=62,
        thickness=5,
        start_angle=0,
        end_angle=360
    },
    {
        name='time',
        arg='%d',
        max=31,
        bg_colour=0xffffff,
        bg_alpha=0.1,
        fg_colour=0x3399cc,
        fg_alpha=0.8,
	change_color=4,	
	start_alpha=0.8,
	end_alpha=0.8,
	startcolor=0x3399cc,
	endcolor=0x3399cc,
        x=100, y=170,
        radius=68,
        thickness=5,
        start_angle=-135,
        end_angle=135
    },
    {
        name='time',
        arg='%m',
        max=12,
        bg_colour=0xffffff,
        bg_alpha=0.1,
        fg_colour=0x3399cc,
        fg_alpha=1,
	change_color=4,	
	start_alpha=1,
	end_alpha=1,
	startcolor=0x3399cc,
	endcolor=0x3399cc,
        x=100, y=170,
        radius=74,
        thickness=5,
        start_angle=-90,
        end_angle=90
    },
    {
        name='cpu',
        arg='cpu1',
        max=100,
        bg_colour=0xffffff,
        bg_alpha=0.2,
        fg_colour=0xffff00,
        fg_alpha=0.4,
	change_color=6,	
	start_alpha=0,
	end_alpha=1,
	startcolor=0xff0000,
	endcolor=0xffffff,
        x=70, y=370,
        radius=25,
        thickness=5,
        start_angle=-90,
        end_angle=180
    },
    {
        name='cpu',
        arg='cpu2',
        max=100,
        bg_colour=0xffffff,
        bg_alpha=0.2,
        fg_colour=0xffff00,
        fg_alpha=0.4,
	change_color=6,	
	start_alpha=0,
	end_alpha=1,
	startcolor=0xff0000,
	endcolor=0xffff00,
        x=125, y=370,
        radius=25,
        thickness=5,
        start_angle=90,
        end_angle=-180
    },
    {
        name='memperc',
        arg='',
        max=100,
        bg_colour=0xffffff,
        bg_alpha=0.2,
        fg_colour=0x33ccff,
        fg_alpha=0.8,
	change_color=6,	
	start_alpha=0,
	end_alpha=1,
	startcolor=0x33ccff,
	endcolor=0x33ccff,
        x=97, y=420,
        radius=25,
        thickness=5,
        start_angle=-90,
        end_angle=180
    },
    {
        name='swapperc',
        arg='',
        max=100,
        bg_colour=0xffffff,
        bg_alpha=0.2,
        fg_colour=0x33ccff,
        fg_alpha=0.8,
	change_color=4,	
	start_alpha=0.8,
	end_alpha=0.8,
	startcolor=0x33ccff,
	endcolor=0x33ccff,
        x=153, y=420,
        radius=25,
        thickness=5,
        start_angle=90,
        end_angle=-180
    },
    {
        name='fs_used_perc',
        arg='/',
        max=100,
        bg_colour=0xffffff,
        bg_alpha=0.2,
        fg_colour=0x33FFFF,
        fg_alpha=0.7,
	change_color=4,	
	start_alpha=0.7,
	end_alpha=0.7,
	startcolor=0x33ffff,
	endcolor=0x33ffff,
        x=125, y=470,
        radius=25,
        thickness=5,
        start_angle=-90,
        end_angle=180
    },
        {
        name='wireless_link_qual_perc',
        arg='wlan0',
        max=100,
        bg_colour=0xFFFFFF,
        bg_alpha=0.2,
        fg_colour=0x33FFff,
        fg_alpha=0.5,
	change_color=6,	
	start_alpha=0,
	end_alpha=0.9,
	startcolor=0x33ffff,
	endcolor=0xff00ff,
        x=153, y=520,
        radius=25,
        thickness=5,
        start_angle=-90,
        end_angle=180
    },
        {
        name='battery_percent',
        arg='',
        max=100,
        bg_colour=0xFFFFFF,
        bg_alpha=0.4,
        fg_colour=0xFF0000,
        fg_alpha=0.3,
	change_color=3,	
	start_alpha=0.8,
	end_alpha=0.3,
	startcolor=0xff0000,
	endcolor=0xff0000,
        x=170, y=607,
        radius=25,
        thickness=5,
        start_angle=-90,
        end_angle=180
    },
        {
        name='time',
        arg='%H',
        max=24,
        bg_colour=0xFFFFFF,
        bg_alpha=0.1,
        fg_colour=0x000000,
        fg_alpha=1,
	change_color=6,	
	start_alpha=0.2,
	end_alpha=0.5,
	startcolor=0xFFFFFF,
	endcolor=0x000000,
        x=53, y=607,
        radius=60,
        thickness=10,
        start_angle=-45,
        end_angle=90
    },
}

-- Use these settings to define the origin and extent of your clock.

clock_r=65

-- "clock_x" and "clock_y" are the coordinates of the centre of the clock, in pixels, from the top left of the Conky window.

clock_x=100
clock_y=170

show_seconds=true

require 'cairo'

function rgb_to_r_g_b(colour,alpha)
    return ((colour / 0x10000) % 0x100) / 255., ((colour / 0x100) % 0x100) / 255., (colour % 0x100) / 255., alpha
end

function draw_ring(cr,t,pt)
    local w,h=conky_window.width,conky_window.height
    
    local xc,yc,ring_r,ring_w,sa,ea=pt['x'],pt['y'],pt['radius'],pt['thickness'],pt['start_angle'],pt['end_angle']
    local bgc, bga, fgc, fga=pt['bg_colour'], pt['bg_alpha'], pt['fg_colour'], pt['fg_alpha']
	local cr1,t1,pt1=cr,t,pt
	local set_o=pt['change_color']
    local stc,edc=pt['startcolor'],pt['endcolor']
    local t_color=edc-stc
    local d_c=t*t_color+stc
	local sta,eda=pt['start_alpha'],pt['end_alpha']
	local t_alpha=eda-sta
	local d_a=t*t_alpha+sta
    local angle_0=sa*(2*math.pi/360)-math.pi/2
    local angle_f=ea*(2*math.pi/360)-math.pi/2
    local t_arc=t*(angle_f-angle_0)

    -- Draw background ring
	if angle_0>angle_f then
	cairo_arc_negative(cr,xc,yc,ring_r,angle_0,angle_f)
	else	
	cairo_arc(cr,xc,yc,ring_r,angle_0,angle_f)
	end
    cairo_set_source_rgba(cr,rgb_to_r_g_b(bgc,bga))
    cairo_set_line_width(cr,ring_w)
    cairo_stroke(cr)
    
    -- Draw indicator ring
	if angle_0>angle_f then
	cairo_arc_negative(cr,xc,yc,ring_r,angle_0,angle_0+t_arc)
	else
    	cairo_arc(cr,xc,yc,ring_r,angle_0,angle_0+t_arc)
	end
	if set_o==3 then
	cairo_set_source_rgba(cr,rgb_to_r_g_b(d_c,d_a))	
	else
		if set_o==2 then
		cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,d_a))
				
		else
			if set_o==1 then
			cairo_set_source_rgba(cr,rgb_to_r_g_b(d_c,fga))	
			else
				if set_o==0 then
				cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))				
				else
					if set_o==4 then			
					change_clr_alpha(cr1,t1,pt1,1)				
					else
						if set_o==5 then
						change_clr_alpha(cr1,t1,pt1,2)
						else
						change_clr_alpha(cr1,t1,pt1,3)
						end
					end
				end
			end
		end
	end
    --cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
    cairo_stroke(cr)        
end

function draw_clock_hands(cr,xc,yc)
    local secs,mins,hours,secs_arc,mins_arc,hours_arc
    local xh,yh,xm,ym,xs,ys
	local mych,mycm,mycs
	local myhc,mymc,mysc
    
    secs=os.date("%S")    
    mins=os.date("%M")
    hours=os.date("%I")
	mych=hours/12
	mycm=mins/60
	mycs=secs/60        

	myhc=mych*(0x3399ff)

    secs_arc=(2*math.pi/60)*secs
    mins_arc=(2*math.pi/60)*mins+secs_arc/60
    hours_arc=(2*math.pi/12)*hours+mins_arc/12
	        

    -- Draw hour hand
    
    xh=xc+0.7*clock_r*math.sin(hours_arc)
    yh=yc-0.7*clock_r*math.cos(hours_arc)
    cairo_move_to(cr,xc,yc)
    cairo_line_to(cr,xh,yh)
    
    cairo_set_line_cap(cr,CAIRO_LINE_CAP_ROUND)
    cairo_set_line_width(cr,5)
	--cairo_set_source_rgba(cr,rgb_to_r_g_b(colour,alpha))
    cairo_set_source_rgba(cr,1,1,1,0.4)
    cairo_stroke(cr)
    
    -- Draw minute hand
    
    xm=xc+0.9*clock_r*math.sin(mins_arc)
    ym=yc-0.9*clock_r*math.cos(mins_arc)
    cairo_move_to(cr,xc,yc)
    cairo_line_to(cr,xm,ym)
    
    cairo_set_line_width(cr,3)
    cairo_stroke(cr)
    --cairo_new_path(cr)
    -- Draw seconds hand
    
    if show_seconds then
        xs=xc+clock_r*math.sin(secs_arc)
        ys=yc-clock_r*math.cos(secs_arc)
        cairo_move_to(cr,xc,yc)
        cairo_line_to(cr,xs,ys)
    
        cairo_set_line_width(cr,1)
        cairo_stroke(cr)
    end
cairo_destroy(cr)
end

function change_clr_alpha(cr,t,pt,num1)

    local w,h=conky_window.width,conky_window.height
    
    local xc,yc,ring_r,ring_w,sa,ea=pt['x'],pt['y'],pt['radius'],pt['thickness'],pt['start_angle'],pt['end_angle']
    local bgc, bga, fgc, fga=pt['bg_colour'], pt['bg_alpha'], pt['fg_colour'], pt['fg_alpha']
	local n1=pt['max']
	if n1<60 then
	n1=60 		--СֶΪ60֤ƽ
	end	
	local set_o=pt['change_color']
    local stc,edc=pt['startcolor'],pt['endcolor']
    local t_color=tonumber(edc-stc)
    local d_c=t*t_color+stc
	local sta,eda=pt['start_alpha'],pt['end_alpha']
	local t_alpha=eda-sta
	local d_a=t*t_alpha+sta
    local angle_0=sa*(2*math.pi/360)-math.pi/2
    local angle_f=ea*(2*math.pi/360)-math.pi/2
    local t_arc=t*(angle_f-angle_0)
	cairo_new_path(cr)
--߿򲿷
	if num1==1 or num1==3 then

	if angle_0>angle_0+t_arc then
	cairo_arc_negative(cr,xc,yc,ring_r+ring_w/2,angle_0,angle_0+t_arc)	
	cairo_line_to(cr,xc+(ring_r-ring_w/2)*math.cos(angle_0+t_arc),yc+(ring_r-ring_w/2)*math.sin(angle_0+t_arc))
        
	cairo_arc(cr,xc,yc,ring_r-ring_w/2,angle_0+t_arc,angle_0)
        cairo_line_to(cr,xc+(ring_r+ring_w/2)*math.cos(angle_0),yc+(ring_r+ring_w/2)*math.sin(angle_0))
	cairo_set_source_rgba(cr,1,1,1,0.5) --߿ɫ
    	cairo_set_line_width(cr,1)
	cairo_stroke_preserve(cr)

    	cairo_stroke(cr)

	else	
	cairo_arc(cr,xc,yc,ring_r+ring_w/2,angle_0,angle_0+t_arc)	
	cairo_line_to(cr,xc+(ring_r-ring_w/2)*math.cos(angle_0+t_arc),yc+(ring_r-ring_w/2)*math.sin(angle_0+t_arc))
        
	cairo_arc_negative(cr,xc,yc,ring_r-ring_w/2,angle_0+t_arc,angle_0)
        cairo_line_to(cr,xc+(ring_r+ring_w/2)*math.cos(angle_0),yc+(ring_r+ring_w/2)*math.sin(angle_0))
	cairo_set_source_rgba(cr,1,1,1,0.5) --߿ɫ
    	cairo_set_line_width(cr,1)
	cairo_stroke_preserve(cr)

    	cairo_stroke(cr)

	end

	end 
--䲿
	if num1==2 or num1==3 then
	cairo_new_path(cr)
	local angle_k,d_k=angle_0,stc
	local sp_angle= (angle_f-angle_0)/n1 --t_arc/n
	local f_angle=t*n1
	local r_k,g_k,b_k,a_k=rgb_to_r_g_b(d_k)
	local a_k=sta
	local r_ke,g_ke,b_ke=rgb_to_r_g_b(edc)
	local a_ke=eda
	local sp_dr,sp_dg,sp_db,sp_a=(r_ke-r_k)/n1,(g_ke-g_k)/n1,(b_ke-b_k)/n1,(a_ke-a_k)/n1

        for i=1,f_angle do
		if angle_k<angle_k+sp_angle then
      		cairo_arc(cr,xc,yc,ring_r,angle_k,angle_k+sp_angle)
		else
		cairo_arc_negative(cr,xc,yc,ring_r,angle_k,angle_k+sp_angle)
		end
		cairo_set_source_rgba(cr,r_k,g_k,b_k,a_k)
		cairo_set_line_width(cr,ring_w-2)
		angle_k=angle_k+sp_angle
		r_k=r_k+sp_dr
		g_k=g_k+sp_dg
		b_k=b_k+sp_db
		a_k=a_k+sp_a
		cairo_stroke(cr)
		cairo_new_path(cr)
        end
	end
end

function conky_clock_rings()
    local function setup_rings(cr,pt)
        local str=''
        local value=0
        
        str=string.format('${%s %s}',pt['name'],pt['arg'])
        str=conky_parse(str)
        
        value=tonumber(str)
	if ( value == nil ) then value = 0 end
        pct=value/pt['max']
        
        draw_ring(cr,pct,pt)
--change_clr_alpha(cr,pct,pt)
    end
    
    -- Check that Conky has been running for at least 5s

    if conky_window==nil then return end
    local cs=cairo_xlib_surface_create(conky_window.display,conky_window.drawable,conky_window.visual, conky_window.width,conky_window.height)
    
    local cr=cairo_create(cs)    
    
    local updates=conky_parse('${updates}')
    update_num=tonumber(updates)
    
    if update_num>5 then
        for i in pairs(settings_table) do
            setup_rings(cr,settings_table[i])
		cairo_stroke(cr)

        end
    end
    
    draw_clock_hands(cr,clock_x,clock_y)
end

